from milk.supervised import randomforest
from milk.supervised.multi import one_against_one
from milk import nfoldcrossvalidation
from milk.supervised import featureselection
import numpy


BUFFER_SIZE = 64000*1024

GT_USERS = 160

def get_GT_locations():
        '''
        Get ground truth locations
        '''
        fr_Home = open(('%d_GT_the_labels_Home.txt')%(GT_USERS), 'r')
        fr_Work = open(('%d_GT_the_labels_Work.txt')%(GT_USERS), 'r')

        the_homes = {}
        the_works = {}  

        for line in fr_Home:
                user, location, p_type = line.strip().split('\t')
                the_homes[user] = location

        for line in fr_Work:
                user, location, p_type = line.strip().split('\t')
                the_works[user] = location

        fr_Home.close()
        fr_Work.close()

        return the_homes, the_works



def get_classifier(filename, num_features):
        '''
        Train a classifier for the Home or Work location
        '''

        ''' Load Data '''
        imported_array = numpy.loadtxt(filename, delimiter = ',')
        features = imported_array[:, 0:num_features-1]
        labels = imported_array[:, num_features-1]

        ''' Train Data '''
        rf_learner = randomforest.rf_learner()
        learner = one_against_one(rf_learner)
        model = learner.train(features, labels)

        return model, learner




def compute_accurancy(filename, num_features, learner):
        '''
        Train a classifier and compute its accuracy !
        '''
        ''' Load Data '''
        imported_array = numpy.loadtxt(filename, delimiter = ',')
        features = imported_array[:, 0:num_features-1]
        labels = imported_array[:, num_features-1]      

        print 'Cross Validation' 
        cmat,names, preds = nfoldcrossvalidation(features, labels, classifier=learner, return_predictions=1)
        
        print cmat
        print 'Total Accuracy:', cmat.trace() /float(cmat.sum())




def classify(cluster, home_model, work_model):
        '''
        Give a string representing a location, extract its features, and use the
        home and work classifiers.
        '''
        lat_lng, features = cluster.strip().split('\t')
        atr1, atr2, atr3, atr4, atr5 = features.split(',')
        duration,rank = map(int, [atr2, atr3])
        tower_days, home_events, work_events = map(float, [atr1, atr4, atr5])
        feature_vector = [tower_days, duration,rank, home_events, work_events]

        #print home_model.apply(feature_vector), work_model.apply(feature_vector)               

        return home_model.apply(feature_vector), work_model.apply(feature_vector)       




def get_classified_instances(fr, home_data_set, work_data_set, num_features):
        '''
        Given a file of users and their clusters described by some features, this method 
        would return two dictionaries containing the locations classified as home or as work 
        '''

        print 'Model for Home'
        home_model, home_learner = get_classifier(home_data_set, num_features)
        #compute_accurancy(home_data_set, num_features, home_learner)   

        print '\n\n'    

        print 'Model for Work'
        work_model, work_learner = get_classifier(work_data_set, num_features)
        #compute_accurancy(work_data_set, num_features, work_learner)   

        classified_homes = {}
        classified_works = {}


        users_classified = 0
        printing_step = 1000
        for line in fr:
                id_str, num_str = line.strip().split('\t')
                locations = int(num_str)
                classified_homes[id_str] = set()
                classified_works[id_str] = set()
                for cluster in fr:
                        lat_lng, features = cluster.strip().split('\t')
                        isHome, isWork = classify(cluster, home_model, work_model)

                        if isHome:
                                classified_homes[id_str].add(lat_lng)
                        if isWork:
                                classified_works[id_str].add(lat_lng)
                        locations -= 1
                        if locations == 0:
                                break
                users_classified += 1

                if (users_classified % printing_step) == 0:
                        print users_classified, ' people classified'            


        return classified_homes, classified_works
        


''' Determine if a location is home and work just by taking the greater one with the home hours and taking the greater one with the work hours '''

def get_home_work_events(cluster):
        '''
        Give a string representing a location, extract its features, and classify it as home or work based on the hours.
        '''
        lat_lng, features = cluster.strip().split('\t')
        atr1, atr2, atr3, atr4, atr5 = features.split(',')
        duration,rank = map(int, [atr2, atr3])
        tower_days, home_events, work_events = map(float, [atr1, atr4, atr5])

        return home_events, work_events



def get_determined_instances(fr):
        '''
        Given a file of users and their clusters described by some features, this method 
        would return two dictionaries containing the locations classified as home or as work 
        '''

        classified_homes = {}
        classified_works = {}

        users_classified = 0
        printing_step = 1000
        for line in fr:
                id_str, num_str = line.strip().split('\t')
                locations = int(num_str)
                classified_homes[id_str] = set()
                classified_works[id_str] = set()
                tmp_home = {}
                tmp_work = {}
                for cluster in fr:
                        lat_lng, features = cluster.strip().split('\t')
                        home_events, work_events = get_home_work_events(cluster)
                        tmp_home [home_events] = lat_lng
                        tmp_work [work_events] = lat_lng
                        locations -= 1
                        if locations == 0:
                                break
                classified_homes[id_str].add(tmp_home[max(tmp_home.keys())])
                classified_works[id_str].add(tmp_work[max(tmp_work.keys())])
                users_classified += 1

                if (users_classified % printing_step) == 0:
                        print users_classified, ' people classified'            


        return classified_homes, classified_works


if __name__ == '__main__':

        num_features = 6
        home_data_set = 'pervasive2011_Home.txt'
        work_data_set = 'pervasive2011_Work.txt'
        #users = ('%s_GT_Extracted_Cluster_Features.txt')%(GT_USERS)
        #users = 'Extracted_Cluster_Features.txt'
        users = "5682_GT_Extracted_Cluster_Features.txt"

        fw_home_work = open('home_2_work', 'w')       



        print 'End of training, Classification Started'


        fr = open(users, 'r', BUFFER_SIZE)
        #classified_homes, classified_works =  get_determined_instances(fr)
        classified_homes, classified_works = get_classified_instances(fr, home_data_set, work_data_set, num_features)
        fr.close
        
        
        one_home_one_work = 0
        mis_classified = 0
        for user in classified_homes:
                if len(classified_homes[user]) == 1 & len(classified_works[user]) == 1:
                        home = classified_homes[user].pop()
                        if home not in classified_works[user]:
                                one_home_one_work += 1
                                fw_home_work.write(('%s\t%s\t%s\n')%(user,home, classified_works[user].pop()))
                        else:
                                mis_classified += 1

                else :
                        mis_classified += 1
                
                
        print 'One home, one Work:', one_home_one_work
        print 'Not found Home or Work:', mis_classified

        correct_home = 0
        for user in classified_homes:
                if len(classified_homes[user]) == 1:
                        correct_home += 1
        print 'Identifying Home: ', correct_home

        correct_work = 0
        for user in classified_works:
                if len(classified_works[user]) == 1:
                        correct_work += 1
        print 'Identifying Work: ', correct_work 


        '''
        Determine the success rate of the classifier
        the_homes, the_works = get_GT_locations()

        
        correct = 0
        home_success = 0
        work_success = 0
        
        for user in classified_homes:
                if len(classified_homes[user]) == 1 & len(classified_works[user]) == 1:
                        home = classified_homes[user].pop()
                        work = classified_works[user].pop()
                        if the_homes[user].find(home) >= 0 & the_works[user].find(work)>=0:
                                correct +=1
                                home_success += 1
                                work_success += 1 
                        else:
                                print 'Missclassified:', user
                                if the_homes[user].find(home) >= 0:
                                        home_success += 1
                                elif the_works[user].find(work)>=0:
                                        work_success += 1
                else:
                        print 'Missclassified:', user
                        home = classified_homes[user].pop()
                        work = classified_works[user].pop()
                        if the_homes[user].find(home) >= 0:
                                home_success += 1
                        elif the_works[user].find(work)>=0:
                                work_success += 1

        print 'Correct:', correct
        print 'Home Success', home_success
        print 'Work Success', work_success
        '''










                


