package es.tid.haewoon.cdr.perturbation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import es.tid.haewoon.cdr.util.CDR;
import es.tid.haewoon.cdr.util.CDRUtil;
import es.tid.haewoon.cdr.util.Constants;

public class CountCallsNearGoogleRoute {
    Logger logger = Logger.getLogger(CountCallsNearGoogleRoute.class);
    
    // TODO We should adjust all paths of reading files
    public static void main(String[] args) throws IOException, ParseException {
        (new CountCallsNearGoogleRoute()).run(Constants.RESULT_PATH + File.separator + "20_calls_near_google_route");
    }
    
    int THRESHOLD = 0;
    
    private void run(String targetDirectory) throws IOException, ParseException {
        boolean success = (new File(targetDirectory)).mkdir();
        if (success) {
            logger.debug("A directory [" + targetDirectory + "] is created...");
        }
        
        List<File> files = CDRUtil.loadFiles(Constants.RESULT_PATH + File.separator + "18_BTS_near_google_routes", ".*_BTS_by.*txt$");
        
        String line;
        
        for (File file : files) {
            String number = file.getName().split("_")[0];
            String means = file.getName().split("_")[3].substring(0, file.getName().split("_")[3].length()-4);
            logger.debug(file + ", " + number + ", " + means);
            
            BufferedReader br = new BufferedReader(new FileReader(file));
            while ((line = br.readLine()) != null) {
                String[] tokens = line.split("\t");
                if (tokens.length != 3) {
                    continue;       // no path from Google
                }
                
                String pathID = tokens[0];
                double btsCount = Double.valueOf(tokens[1]);
                List<String> BTSs = Arrays.asList(tokens[2].split("\\|"));
                
                try {
                    checkCoverage("training", number, means, pathID, BTSs, targetDirectory);
                    checkCoverage("test", number, means, pathID, BTSs, targetDirectory);
                } catch (FileNotFoundException fnfe) {
                    logger.error(number, fnfe);
                }
            }
        }
    }
    
    private void checkCoverage(String relativeDirectory, String number, String means, String pathID, List<String> BTSs, String targetDirectory) throws IOException, ParseException {
        String baseRead = Constants.RESULT_PATH + File.separator + "19_train_and_test_route";
        String line;
        BufferedReader br = new BufferedReader(new FileReader(baseRead + File.separator + relativeDirectory + File.separator + number));
        List<String> matched = new ArrayList<String>();
        while ((line = br.readLine()) != null) {
            CDR cdr = new CDR(line);
            String initBTS = CDRUtil.getCell(cdr.getInitCellID()).getBTSID();

            if (BTSs.contains(initBTS)) {
                matched.add(initBTS);
            }
            
            String finBTS = CDRUtil.getCell(cdr.getFinCellID()).getBTSID();
            if (!initBTS.equals(finBTS)) {
                matched.add(finBTS);
            }
        }
        Set<String> uniq_matched = new HashSet<String>(matched);
        
        BufferedWriter bw = new BufferedWriter(new FileWriter(targetDirectory + File.separator + number + "_" + relativeDirectory + ".txt", true));
        bw.write(means + "\t" + pathID + "\t" + matched.size() + "\t" + uniq_matched.size());
        bw.newLine();
        bw.close();
    }   
}
