package es.tid.haewoon.cdr.perturbation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

import org.apache.log4j.Logger;

import es.tid.haewoon.cdr.util.Constants;

public class CreateRandomPerturbations {
    Logger logger = Logger.getLogger(CreateRandomPerturbations.class);
    
    public static void main(String[] args) throws IOException {
        (new CreateRandomPerturbations()).run(Constants.RESULT_PATH + File.separator + "16_perturbations");
    }
    
    int NUMBER_OF_PERTURBATIONS = 100;
    
    public void run(String targetDirectory) throws IOException {
        boolean success = (new File(targetDirectory)).mkdir();
        if (success) {
            logger.info("A directory [" + targetDirectory + "] is created");
        }
        
        BufferedReader br = new BufferedReader(new FileReader(
                Constants.RESULT_PATH + File.separator + "11_home_work_lat_long" + File.separator + "home_2_work"));
        String line;
        
        Random random = new Random();
        
        while ((line = br.readLine()) != null) {
            String[] tokens = line.split("\t");
            
            String telephone = tokens[0];
            String home = tokens[1];
            String work = tokens[2];
            
            BufferedWriter bw = new BufferedWriter(new FileWriter(targetDirectory + File.separator + telephone + ".txt"));
            bw.write("0\t" + home + "\t" + work);
            bw.newLine();
            for (int i = 1; i < NUMBER_OF_PERTURBATIONS; i++) {
                // within the circle of which radius is 1,000m
                String new_home = perturbate(home, random.nextInt(360), 1000);
                String new_work = perturbate(work, random.nextInt(360), 1000);
                
                bw.write(i + "\t" + new_home + "\t" + new_work);
                bw.newLine();
            }
            bw.close();
        }
    }
    
    private String perturbate(String location, int bearing, double distance) {
        double R = 6378137.0;
        
        double lat = Double.valueOf(location.split(",")[0]);
        double lon = Double.valueOf(location.split(",")[1]);
        
        double latA = Math.toRadians(lat);
        double lonA = Math.toRadians(lon);

        double angularDistance = distance / R;
        double trueCourse = Math.toRadians(bearing);

        double new_lat = Math.asin(
                Math.sin(latA) * Math.cos(angularDistance) + 
                Math.cos(latA) * Math.sin(angularDistance) * Math.cos(trueCourse));

        double dlon = Math.atan2(
                Math.sin(trueCourse) * Math.sin(angularDistance) * Math.cos(latA), 
                Math.cos(angularDistance) - Math.sin(latA) * Math.sin(lat));
        double new_lon = ((lonA + dlon + Math.PI) % (2*Math.PI)) - Math.PI;
        
        return Math.toDegrees(new_lat) + "," + Math.toDegrees(new_lon);
    }
}
