package es.tid.haewoon.cdr.perturbation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import es.tid.haewoon.cdr.util.BTS;
import es.tid.haewoon.cdr.util.CDRUtil;
import es.tid.haewoon.cdr.util.Constants;
import es.tid.haewoon.cdr.util.Province;

public class FindBTSNearGoogleRoute {
    Logger logger = Logger.getLogger(FindBTSNearGoogleRoute.class);
    double THRESHOLD = 100;    // meter 
    Set<BTS> BTSs = new HashSet<BTS>(); 
    
    public static void main(String[] args) throws IOException {
        (new FindBTSNearGoogleRoute()).run(Constants.RESULT_PATH + File.separator + "18_BTS_near_google_routes");
    }
    
    private void run(String targetDirectory) throws IOException {
        boolean success = (new File(targetDirectory)).mkdir();
        if (success) {
            logger.info("A directory[" + targetDirectory + "] is created");
        }
        
//        findNearPath(targetDirectory, "by_car");
        findNearPath(targetDirectory, "by_transit");
        findNearPath(targetDirectory, "by_walk");
    }
    
    private void findNearPath(String targetDirectory, String postfix) throws IOException {
        List<File> files = CDRUtil.loadFiles(Constants.RESULT_PATH + File.separator + "17_google_route", ".*_path_" + postfix + ".txt$");
        for (File file : files) {
            BufferedReader br = new BufferedReader(new FileReader(file));
            BufferedWriter bw = new BufferedWriter(new FileWriter(targetDirectory + File.separator + file.getName().split("_")[0] + "_BTS_" + postfix + ".txt"));
            String line;
            while ((line = br.readLine()) != null) {
                logger.debug(file.getName() + ", " + line);
                BTSs.clear();
    
                String[] tokens = line.split("\t");
                String number = tokens[0];
                
                try {
                    String hw_route = tokens[1];
                    load(hw_route);
                } catch (ArrayIndexOutOfBoundsException aioobe) {
                    logger.error(aioobe);
                }

                try {
                    String wh_route = tokens[2];
                    load(wh_route);
                } catch (ArrayIndexOutOfBoundsException aioobe) {
                    logger.error(aioobe);
                }
                bw.write(number + "\t" + BTSs.size() + "\t" + CDRUtil.join(BTSs, "|"));
                bw.newLine();

            }
            bw.close();
        }
    }
    
    private boolean load(String path) throws IOException {
        String line;
        boolean isFirst = true;
        double[] ll = {0.0, 0.0};
        String[] locations = path.split("\\|");
        for (String location : locations) {
            try {
                String[] tokens = location.split(",");
                double[] cl = {Double.valueOf(tokens[0]), Double.valueOf(tokens[1])};
    
                checkPivot(cl[0], cl[1]);
                
                if (!isFirst) {
                    checkPathBetweenPivots(ll[0], ll[1], cl[0], cl[1]);
                } else {
                    isFirst = false;
                }
                
                ll[0] = cl[0];
                ll[1] = cl[1];
            } catch (Exception e) {
                logger.error(location, e);
                return true;
            }
        }
        
        return true;
        
    }
    
    private void checkPivot(double clat, double clon) {
        for (BTS bts : CDRUtil.getBTSs(Province.BARCELONA)) {
            if (bts.closeEnough(clat, clon, THRESHOLD)) {
//                logger.debug(bts + " near " + clat + "," + clon);
                BTSs.add(bts);
            }
        }
    }
    
    private void checkPathBetweenPivots(double llat, double llon, double clat, double clon) {
        final double latDelta = clat - llat;
        final double lonDelta = clon - llon;
        
        if (latDelta == 0 && lonDelta == 0) {
            return;
        }
        
        for (BTS bts : CDRUtil.getBTSs(Province.BARCELONA)) {
            final double u = ((bts.getLatitude() - llat) * latDelta + (bts.getLongitude() - llon) * lonDelta) / (latDelta * latDelta + lonDelta * lonDelta);
            
            if (u < 0 || u > 1) {
                continue;
            } else {
                double[] closePoint = {llat + u * latDelta, llon + u * lonDelta};
                if (bts.closeEnough(closePoint[0], closePoint[1], THRESHOLD)) {
//                    logger.debug(bts + " near " + llat + "," + llon + "->" + clat + "," + clon);
                    BTSs.add(bts);
                }
            }
        }
    }
}
